/******************************************************************************
 * 
 * \file      main.c
 * \author    Jesús Martin palomino Velezaquez 
 * \date      Radio FM with TEA5767 and W806
 * \note      code of lcd 2x16 based on the library for stm32f103xx family
 * \note      
 *  
******************************************************************************/             

#define TEA_ADDR (0x60 << 1)

#include <stdio.h>
#include "wm_hal.h"
#include "LCD_2x16.h"
#include "gpio.h"


LCD_HandleTypeDef lcd;

I2C_HandleTypeDef hi2c;

void Error_Handler(void);

float fre1=100.9;

static void I2C_Init(void)
{
	hi2c.Instance = I2C;
    hi2c.Frequency = 400000;
    I2C_Init_Only_Write(&hi2c);

}

void Write_Frequency(uint8_t dev_addr, float frequency) {
	
	short int i=0; 
	unsigned int frequencyB = 4 * (frequency * 1000000 + 225000) / 32768;
	uint8_t dato[5];
	
	    dato[0] = frequencyB >> 8;
	    dato[1] = frequencyB & 0XFF;
		dato[2] = 0xB0;
		dato[3] = 0x10;
		dato[4] = 0x00;
		
	I2C->DATA = dev_addr;    // enter the address into the data register MSP4725
	I2C->CR_SR = I2C_CR_WR | I2C_CR_START;  // turn on the module for transmission and emission
	while(I2C->CR_SR & I2C_SR_TIP){};// waiting for the end of the transfer

	for(i=0; i<=4; i++) {
		
		I2C->DATA = dato[i];  // enter the address of the register into the data register
        I2C->CR_SR = I2C_CR_WR ;        // send a byte
        while(I2C->CR_SR & I2C_SR_TIP){}; // waiting for the end of the transfer
	
    }  
      I2C->DATA = dev_addr+1;  //transfer the mcp4725 address  with a sign of reading
	  I2C->CR_SR = I2C_CR_WR | I2C_CR_START ; // pass a byte and restart to start reading
	  while(I2C->CR_SR & I2C_SR_TIP){}; // waiting for bus release
	  I2C->CR_SR = I2C_CR_RD |  I2C_CR_ACK | I2C_CR_STOP ; // receive a byte and immediately after it - NACK and STOP
	  while (I2C->CR_SR & I2C_SR_BUSY); // wait for bus release
	 // here you can assign the received byte to some variable by reading I2C->DATA
     // some_variable = I2C->DATA; 
}

int main(void)
{
  
 SystemClock_Config(CPU_CLK_160M);
   
  HAL_Init();
  GPIO_Init();
  I2C_Init(); 
  
  lcd.RS.Port = GPIOB;      
  lcd.RS.Pin = GPIO_PIN_15;
  
  lcd.EN.Port = GPIOB;
  lcd.EN.Pin = GPIO_PIN_10;
  
  lcd.DB4.Port = GPIOB;
  lcd.DB4.Pin = GPIO_PIN_11;
  
  lcd.DB5.Port = GPIOB;
  lcd.DB5.Pin = GPIO_PIN_16;
  
  lcd.DB6.Port = GPIOB;
  lcd.DB6.Pin = GPIO_PIN_17;
  
  lcd.DB7.Port = GPIOB;
  lcd.DB7.Pin = GPIO_PIN_18;
	
  
  HAL_Delay(100);
  LCD_sendCmd(&lcd, 0x33); // Initialize controller
  LCD_sendCmd(&lcd, 0x32); // Set 4-bit mode
  LCD_sendCmd(&lcd, 0x28); // 4 bit, 2 line, 5x7
  LCD_sendCmd(&lcd, 0x06); // Cursor direction -> right
  LCD_sendCmd(&lcd, 0x0C); // Display on, cursor off
  LCD_sendCmd(&lcd, clear); // Clear display
  HAL_Delay(100);


 short int state0=1, state1=1; 	
 int line = 0, position = 4;
 
 LCD_sendCmd(&lcd, 0x80 | (line*0x40 + position));
 LCD_TEXT(&lcd,"RADIO_FM");
 line = 1, position = 10;
 LCD_sendCmd(&lcd, 0x80 | (line*0x40 + position));
 LCD_TEXT(&lcd,"MHZ");
 
 Write_Frequency(TEA_ADDR,fre1);
 line = 1, position = 3;
 LCD_sendCmd(&lcd, 0x80 | (line*0x40 + position));
 LCD_FLOAT(&lcd, fre1, 5); 
 
 
 while (1)
  {
	 if(HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_6)==LOW) {  
	 HAL_Delay(10); 
	 if(HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_6)==LOW) state0=0, HAL_Delay(10);
   }
     if( HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_6)==HIGH && state0==0){ 
        fre1=fre1+0.1;
		if (fre1 > 108.0) fre1 = 88.0;
	    if (fre1 < 88.0)  fre1 = 108.0;
        Write_Frequency(TEA_ADDR,fre1);
        line = 1, position = 3;
        LCD_sendCmd(&lcd, 0x80 | (line*0x40 + position));
		LCD_FLOAT(&lcd, fre1, 5); 
		state0=1;
		HAL_Delay(80); 
     }
	 
	 if(HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_7)==LOW) {  
	 HAL_Delay(10); 
	 if(HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_7)==LOW) state1=0, HAL_Delay(10);
   }
     if( HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_7)==HIGH && state1==0){ 
        fre1=fre1-0.1;
		if (fre1 > 108.0) fre1 = 88.0;
	    if (fre1 < 88.0)  fre1 = 108.0;
        Write_Frequency(TEA_ADDR,fre1);
        line = 1, position = 3;
        LCD_sendCmd(&lcd, 0x80 | (line*0x40 + position));
		LCD_FLOAT(&lcd, fre1, 5); 
		state1=1;
        HAL_Delay(80);
     }
  }
   
    
}

void Error_Handler(void)
{
    while (1)
    {
    }
}

void assert_failed(uint8_t *file, uint32_t line)
{
    printf("Wrong parameters value: file %s on line %d\r\n", file, line);
}
